import path from 'node:path';
import fse from 'fs-extra';
import { initConfig, initContext, updateConfig, updateViteUserConfig, } from './config.js';
import { cachebuster } from './cachebuster.js';
import { processAsset } from './processAsset.js';
import { processAssetWithoutHash } from './processAssetWithoutHash.js';
import { processPublic } from './processPublic.js';
import { processCache } from './processCache.js';
const vitePluginImageminCache = (userSettings) => {
    const config = initConfig(userSettings || {});
    const ctx = initContext(config);
    let viteConfig;
    fse.ensureDirSync(config.cacheDir);
    return {
        name: 'vite-plugin-imagemin-cache',
        enforce: 'post',
        apply: 'build',
        config(viteUserConfig) {
            return updateViteUserConfig(ctx, config, viteUserConfig);
        },
        configResolved(resolvedConfig) {
            viteConfig = resolvedConfig;
            updateConfig(config, viteConfig);
        },
        transformIndexHtml(html) {
            if (config.asset.cachebuster)
                return cachebuster(html, config.asset.cachebuster);
            return html;
        },
        async generateBundle(_options, bundle) {
            Object.entries(bundle).forEach(([fileName, asset]) => {
                if (asset.type !== 'asset' ||
                    !asset.name ||
                    !ctx.targetExtentions.has(path.extname(fileName)) ||
                    (ctx.excludeMatcher && ctx.excludeMatcher(asset.name)))
                    return;
                ctx.assetTargets.add(fileName);
            });
            if (ctx.assetTargets.size === 0)
                return;
            console.log('');
            ctx.outputLog('info', 'processing static asset...');
            if (config.asset.useCrc)
                await processAssetWithoutHash(ctx, bundle);
            else
                await processAsset(ctx, viteConfig, bundle);
        },
        async closeBundle() {
            if (ctx.publicDir !== '') {
                ctx.outputLog('info', 'processing public directory...');
                await processPublic(ctx, viteConfig);
            }
            const now = new Date().getTime();
            await processCache(ctx, now);
        },
    };
};
export default vitePluginImageminCache;
