import path from 'node:path';
import fse from 'fs-extra';
import { imageProcessor } from './imageProcessor.js';
export async function processAsset(ctx, viteConfig, bundle) {
    const queue = [...ctx.assetTargets].map(async (fileName) => {
        const { config } = ctx;
        const outDir = viteConfig.build.outDir;
        return ctx.limit(async () => {
            const cachePath = path.resolve(path.join(config.cacheDir, fileName));
            if (config.asset.preventOverwrite &&
                fse.existsSync(path.join(outDir, fileName))) {
                delete bundle[fileName];
                ctx.outputLog('info', 'skip:', fileName);
                return;
            }
            if (fse.existsSync(cachePath)) {
                ;
                bundle[fileName].source =
                    await fse.readFile(cachePath);
                ctx.outputLog('info', 'cache hit:', fileName);
                return;
            }
            const source = bundle[fileName].source;
            const content = await imageProcessor(config.plugins, source);
            if (content === false) {
                ctx.outputLog('error', 'imagemin error:', fileName);
                throw new Error('imagemin failed');
            }
            else {
                ;
                bundle[fileName].source = content;
                await fse.outputFile(cachePath, content);
            }
            ctx.outputLog('info', 'compressed:', fileName, `(processing: ${ctx.limit.activeCount.toString()}, pending: ${ctx.limit.pendingCount.toString()})`);
        });
    });
    await Promise.all(queue);
}
