import path from 'node:path';
import fse from 'fs-extra';
import { crc32 } from 'polycrc';
import { imageProcessor } from './imageProcessor.js';
export async function processAssetWithoutHash(ctx, bundle) {
    const queue = [...ctx.assetTargets].map(async (fileName) => {
        const { config } = ctx;
        return ctx.limit(async () => {
            const cachePath = path.resolve(path.join(config.cacheDir, fileName));
            const source = bundle[fileName].source;
            const checksum = crc32(source);
            ctx.assetHashMap.set(fileName, checksum);
            const cacheData = ctx.cacheDb.getData(fileName);
            if (cacheData &&
                cacheData.checksum === checksum &&
                fse.existsSync(cachePath)) {
                ;
                bundle[fileName].source =
                    await fse.readFile(cachePath);
                ctx.outputLog('info', 'cache hit:', fileName);
                return;
            }
            else {
                const content = await imageProcessor(config.plugins, source);
                if (content === false) {
                    ctx.outputLog('error', 'imagemin error:', fileName);
                    throw new Error(`imagemin failed: ${fileName}`);
                }
                else {
                    ;
                    bundle[fileName].source = content;
                    await fse.outputFile(cachePath, content);
                }
                ctx.outputLog('info', 'compressed:', fileName, `(processing: ${ctx.limit.activeCount.toString()}, pending: ${ctx.limit.pendingCount.toString()})`);
            }
        });
    });
    await Promise.all(queue);
}
