import path from 'node:path';
import fse from 'fs-extra';
export async function processCache(ctx, now) {
    const { config, cacheDb } = ctx;
    cacheDb.countdown();
    if (config.asset.useCrc) {
        for (const [fileName, checksum] of ctx.assetHashMap)
            cacheDb.upsertPublic(fileName, checksum, now);
    }
    else {
        cacheDb.renewAsset(ctx.assetTargets, now);
        cacheDb.insertAsset(ctx.assetTargets, now);
    }
    for (const [fileName, checksum] of ctx.publicHashMap)
        cacheDb.upsertPublic(fileName, checksum, now);
    const expiredFiles = cacheDb.getExpired(now);
    await Promise.all(expiredFiles.map((file) => {
        return fse.remove(path.join(config.cacheDir, file.fileName));
    }));
    cacheDb.removeExpired(now);
    cacheDb.close();
}
