import os from 'node:os';
import path from 'node:path';
import picomatch from 'picomatch';
import { createLogger } from 'vite';
import ansis from 'ansis';
const logger = createLogger();
export function initMatcher(pattern) {
    if (pattern === undefined)
        return false;
    if (typeof pattern === 'string' || Array.isArray(pattern))
        return picomatch(pattern);
    return pattern;
}
export function outputLog(type, green, yellow, dim) {
    return logger[type](ansis.cyanBright('[imagemin-cache]') +
        (green ? ansis.green(` ${green}`) : '') +
        (yellow ? ansis.yellow(` ${yellow}`) : '') +
        (dim ? ansis.dim(` ${dim}`) : ''));
}
export const isWindows = os.platform() === 'win32';
export function slash(p) {
    return p.replace(/\\/g, '/');
}
export function normalizePath(id) {
    return path.posix.normalize(isWindows ? slash(id) : id);
}
export function getResolvedRoot(root) {
    return normalizePath(root ? path.resolve(root) : process.cwd());
}
export function getResolvedPublicDir(publicDir, resolvedRoot) {
    return publicDir !== false && publicDir !== ''
        ? path.resolve(resolvedRoot, typeof publicDir === 'string' ? publicDir : 'public')
        : '';
}
